; File Name   :	MELODIE.BIN
; Format      :	Binary file
; Base Address:	0000h Range: 0000h - 0800h Loaded length: 0800h

; Liedermelodien: Die Speicherung der Tne erfolgt dabei byteweise, wobei ein Byte
; (8 Bits) einen in Tonhhe und -lnge variablen Ton darstellt. Aufbau eines
; Tonbytes: Die unteren vier Bits reprsentieren die Tonhhe, wobei die
; Kombination 0000 eine Ausnahme macht. Diese erzeugt keinen Ton, sondern eine
; Pause. Folglich ergibt die Folge 0001 den tiefsten, 1111 den hchsten Ton. Die
; nchsten drei Bits bestimmen die Tonlnge in Achtelnoten. Hier bedeuten 000 eine
; Achtelnote, 111 acht Achtelnoten. Lngere Tne sind aus mehreren Bytes
; zusammenzusetzen. Das Bit 7 wird als Stop-Bit verwendet. Es kennzeichnet, wenn
; es gesetzt ist, ein Liedende. Folgen zwei Bytes mit gesetztem Stop-Bit
; aufeinander, ist das Ende der Liedertabelle erreicht.
;
; Die Arbeitsweise der Klingel ist etwa folgende: Bei Bettigen des Klingelknopfes
; wird ber das Speicher-Flip-Flop (D42) im Prozessor eine Programmunterbrechung
; erzwungen, welche diesen aus seinem Warteprogramm herausholt. Ein internes
; Register (IY) zeigt bereits auf den Anfang des nchsten Liedes. Bevor mit der
; Erzeugung der Tne begonnen wird, mu zunchst einmal die Anzahl der
; Tonschwingungen fr den nchsten auszugebenden Ton errechnet werden. Dafr
; enthlt das Programm eine Tabelle, die fr jeden der 15 mglichen Tne die
; Schwingungsanzahl fr eine Achtelnote bereithlt. Durch eine einfache
; Multiplikation mit der Anzahl der Achtelnoten steht nun die Zahl der
; auszugebenden Tonschwingungen fest.
;
; Ein raffiniert ausgeklgeltes Programm bernimmt nun die Ausgabe der
; Halbschwingungen ber das Ausgabe-FlipFlop (D1) in zeitlich richtigem Abstand.
; Nach Ausgabe aller Halbschwingungen wird der nchste Ton des Liedes ermittelt
; und ausgegeben, so lange, bis ein gesetztes Stop-Bit erreicht ist. Wird dieses
; auch noch im folgenden Byte erkannt, erfolgt das Rcksetzen des Zeigeregisters
; auf den Anfang des Liedertabelle. Es schliet sich eine Zeitschleife an, die das
; sofortige Auslsen der Klingel am Liedende bei ungeduldigen Gsten verhindert.
; Nun wird das Speicher-Flip-Flop (D4.2) zurckgesetzt, und die Klingel ist zur
; Ausgabe des nchsten Liedes bereit.

		cpu	Z80
		org	0

; ---------------------------------------------------------------------------
; Start mit Liedtabelle
; angesprungen wird diese Adresse ber Stack
loc_0:		ld	iy, song	; Adresse 1. Lied

; Start mit nchstem Lied
; angesprungen wird diese Adresse ber Stack
loc_4:		ld	a, 80h		; D7=1
		out	(c), a		; NMI ausschalten
		halt			; weiter gehts nur mit NMI-Taste -> Sprung zu NMI
; ---------------------------------------------------------------------------
;Ton-Tabelle 1 (Anzahl Tonschwingungen pro Achtelnote)
		dw 234			; Ton 15 (d'')
		dw 221
		dw 209
		dw 197
		dw 186
		dw 176
		dw 166
		dw 156
		dw 148
		dw 139
		dw 132
		dw 124
		dw 117
		dw 110
		dw 104			; Ton 1	(c')
tab1:		dw 234			; Ton 0	(Pause)

;Ton-Tabelle 2 (Sprungadressen)
		dw ton15		; Ton 15 (d'')
		dw ton14
		dw ton13
		dw ton12
		dw ton11
		dw ton10
		dw ton9
		dw ton8
		dw ton7
		dw ton6
		dw ton5
		dw ton4
		dw ton3
		dw ton2
		dw ton1			; Ton 1	(c')
tab2:		dw pause		; Ton 0	(Pause)

;Stack
stack1:		dw loc_0
stack2:		dw loc_4
; ---------------------------------------------------------------------------
;Zeitschleife fr Tonerzeugung

pause:		ld	c, 0
		out	(c), c
		ld	b, 76
pause1:		djnz	pause1
		ld	r, a
		dec	de
		ld	a, d
		or	e
		jp	nz, pause
		jp	kli2
; ---------------------------------------------------------------------------
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
; ---------------------------------------------------------------------------
;Hauptprogramm 
; bei Tastendruck ber NMI ausgelst
		org 66h
NMI:		
; Ton bearbeiten
; Tondauer
		ld	a, (iy+0)	; Ton holen
		and	0Fh		; nur Tonhhe
		rlca			; x 2
		ld	e, a
		ex	af, af'
		ld	a, e
		ex	af, af'
		ld	d, 0
		ld	hl, tab1	;Ton-Tabelle 1 (Tonschwingungen)
		or	a
		sbc	hl, de
		ld	e, (hl)
		inc	hl
		ld	d, (hl)		; DE= = Anzahl Halbschwingungen
;
		ld	a, (iy+0)	; Ton holen
		and	70h		; nur Tondauer
		rrca
		rrca
		rrca
		rrca
		ld	b, a		; B = Tondauer
		inc	b		; + 1
		ld	hl, 0
nmi1:		add	hl, de
		djnz	nmi1
		ex	de, hl		; DE = (Tondauer+1) x Anz. Tonschwingungen
					; die NMI-Schleife wird DE mal durchlaufen,
					; um fr alle Tonhhen die gleiche Tondauer zu erreichen
; Tonhhe
		ex	af, af'
		ld	c, a
		ld	b, 0
		ld	hl, tab2	;Ton-Tabelle 2 (Sprungadressen)
		or	a
		sbc	hl, bc
		ld	c, (hl)
		inc	hl
		ld	b, (hl)
		ld	h, b
		ld	l, c		; HL=Sprungadresse

; Ton-Schleife
beep:		inc	c		; ???
					; 4 Takte
		res	7, c		; D7=0
					; 8 Takte
		out	(c), c		; Ausgabe einer Halbschwingung
					; 12 Takte
		jp	(hl)		; ton_xx starten
					; 4 Takte
; ---------------------------------------------------------------------------
; Takte setzen fr Tonhhe

;d''
ton15:		neg			; 8 Takte
		ld	b, 75
		jp	kli1
; ---------------------------------------------------------------------------

;cis''
ton14:		rld			; 18 Takte
		ld	b, 79
		jp	kli1
; ---------------------------------------------------------------------------

;c''
ton13:		ld	sp, hl		; 20 Takte
		ld	b, 85
		jp	kli1
; ---------------------------------------------------------------------------

;h'
ton12:		ld	(hl), a		; 13 Takte
		ld	b, 90
		jp	kli1
; ---------------------------------------------------------------------------

;ais'
ton11:		ld	sp, hl		; 6 Takte
		ld	b, 96
		jp	kli1
; ---------------------------------------------------------------------------

;a'
ton10:		ld	sp, hl		; 6 Takte
		ld	b, 102
		jp	kli1
; ---------------------------------------------------------------------------

;gis'
ton9:		nop			; 4 Takte
		neg			; 8 Takte
		ld	b, 108
		jp	kli1
; ---------------------------------------------------------------------------

;g'
ton8:		neg			; 8 Takte
		ld	b, 115
		jp	kli1
; ---------------------------------------------------------------------------

;fis'
ton7:		inc	(hl)		; 11 Takte
		ld	b, 122
		jp	kli1
; ---------------------------------------------------------------------------

;f'
ton6:		ld	sp, hl		; 6 Takte
		ld	b, 130
		jp	kli1
; ---------------------------------------------------------------------------

;e'
ton5:		nop			; 4 Takte
		ld	b, 138
		jp	kli1
; ---------------------------------------------------------------------------

;dis'
ton4:		ld	ix, 0		; 19 Takte
		ld	b, 146
		jp	kli1
; ---------------------------------------------------------------------------

;d'
ton3		ld	ix, 0		; 19 Takte
	        ld	b, 155
	        jp	kli1
; ---------------------------------------------------------------------------

;cis'
ton2:		ld	r, a		; 9 Takte
		ld	b, 165
		jp	kli1
; ---------------------------------------------------------------------------

;c'
ton1:		inc	(hl)		; 11 Takte
		ld	b, 175
		jp	kli1

; ---------------------------------------------------------------------------

; Ton erklingen lassen
; Zusammen mit beep b*13 + 60 Takte
kli1:		djnz	kli1		; Zeitschleife Tonhhe
					; 13/8 Takte
		dec	de		; DE = Anzahl Halbschwingungen
					; 6 Takte
		ld	a, e		; 4 Takte
		or	d		; 4 Takte
		jp	nz, beep	; nchste Halbschwingung (28 Takte)
					; 10/10 Takte

; nchsten Ton des Liedes
kli2:		bit	7, (iy+0)	; Lied-Ende ?
		jr	nz, kli3	; ja
		inc	iy		; sonst nchster Ton
		jp	NMI

; ---------------------------------------------------------------------------
; Lied-Ende
kli3:		bit	7, (iy+1)	; Liedtabellen-Ende ?
		jr	nz, kli4
		inc	iy
		ld	sp, stack2	; nein,	nchstes Lied
		jr	kli5
kli4:		ld	sp, stack1	; sonst	in Liedtabelle wieder von vorn beginnen
;
; Zeitschleife 10s warten
kli5:		ld	d, 10		; 10 Sekunden warten
kli6:		ld	bc, 0		; Zeitschleife
		out	(c), c
kli7:		dec	bc
		ld	a, b
		or	c
		jr	nz, kli7
		dec	d
		jr	nz, kli6	; etwa 1 Sekunde
		retn			; -> Sprungadresse von Stack
; ---------------------------------------------------------------------------
; Lied-Tabelle
; Bit7 Stop-bit
; Bit6..4 Tonlnge
; Bit3..0 Tonhhe, 0000=Pause
; Ende: 2x Stop-Bit

song:		equ	$

_ton		eval	0

tonhoehe	macro hoehe,{noexpand}
		if "hoehe"=="PAUSE"
_ton		eval	0
		elseif "hoehe"=="C"
_ton		eval	1
		elseif "hoehe"=="CIS"
_ton		eval	2
		elseif "hoehe"=="D"
_ton		eval	3
		elseif "hoehe"=="DIS"
_ton		eval	4
		elseif "hoehe"=="E"
_ton		eval	5
		elseif "hoehe"=="F"
_ton		eval	6
		elseif "hoehe"=="FIS"
_ton		eval	7
		elseif "hoehe"=="G"
_ton		eval	8
		elseif "hoehe"=="GIS"
_ton		eval	9
		elseif "hoehe"=="A"
_ton		eval	10
		elseif "hoehe"=="AIS"
_ton		eval	11
		elseif "hoehe"=="H"
_ton		eval	12
		elseif "hoehe"=="C1"
_ton		eval	13
		elseif "hoehe"=="CIS1"
_ton		eval	14
		elseif "hoehe"=="D1"
_ton		eval	15
		else
		ERROR	"Falsche Tonhhe: hoehe"
		endif
		endm

ton	macro	hoehe, dauer, pause, {noexpand} 
		tonhoehe hoehe
_ton		eval	_ton+(dauer-1)*16		
		IF      "pause"=="ENDE" 
		db	_ton+080h
		ELSEIF	"pause"<>""
		db	_ton
		db	(pause-1)*16
		ELSE
		db	_ton
		ENDIF
	endm

	include	fa0486.bin.asm
;	include	LIED.HEX.asm
;	include	LIED.LNG.asm
;	include	LIED.WEI.asm
;	include	LIED2.HEX.asm
;	MELODIE.BIN.asm
;	Weihnachts.BIN.asm


		db	080h		; Listenende

; originales Copyright (in fa0486.bin)
; 		db	"DIESES PROGRAMM WURDE VON STEFFEN LEHM"
;		db	"ANN, 1142 BERLIN, BRODOWINER RING 30 G"
;		db	"ESCHRIEBEN. DIE WEITERGABE AN DRITTE I"
;		db	"ST ERLAUBT, WENN DIE URHEBERSCHAFT NIC"
;		db	"HT VERAENDERT ODER VERSCHLEIERT WIRD. "

		end
